package com.androidbook.OpenGL;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import javax.microedition.khronos.opengles.GL10;

import android.content.Context;
import android.opengl.GLES20;

//filename: SimpleTriangleRenderer.java
public class ES20SimpleCubeRenderer extends ES20AbstractRenderer
{
    //A raw native buffer to hold the point coordinates
    private FloatBuffer mFVertexBuffer;
    private static final int FLOAT_SIZE_BYTES = 4;

//front-face: f1,f2,f3,f4
//starting top-right-anti-clockwise    
//f1(1,1) f2(-1,1) f3(-1,-1) f4(1,-1): z plane 0
    
//back-face: b1,b2,b3,b4
//starting bottom-right-anti-clockwise    
//b1(1,-1) b2(1,1) b3(-1,1) b4(-1,-1) : z plane 1
    private float z = 2.0f;
    private final float[] mTriangleVerticesData = {
    		//1. front-triangles
    		//f1,f2,f3
    		1,1,0,  		-1,1,0,    			-1,-1,0,
    		//f1,f3,f4
    		1,1,0,    		-1,-1,0,    		1,-1,0,
    		
    		//2. back-triangles
    		//b1,b2,b3
    		1,-1,z,    		1,1,z,    		-1,1,z,
    		//b3,b4,b1
    		-1,1,z,    		-1,-1,z,    		1,-1,z,
    		
    		//3. right-triangles
    		//b2,f1,f4
    		1,1,z,    		1,1,0,    		1,-1,0,
    		//b2,f4,b1
    		1,1,z,			1,-1,0,			1,-1,z,
    		
    		//4. left-triangles
    		//b3, f2, b4
    		-1,1,z,    		-1,1,0,    		-1,-1,z,
    		//b4 f2 f3
    		-1,-1,z,    		-1,1,0,    		-1,-1,0,
    		
    		//5. top-triangles
    		//b2, b3, f2
    		1,1,z,			-1,1,z,				-1,1,0,
    		//b2, f2, f1
    		1,1,z,			-1,1,0,			1,1,0,
    		
    		//6. bottom-triangles
    		//b1, b4, f3
    		1,-1,z,		-1,-1,z,	-1,-1,0,
    		//b1, f3, f4
    		1,-1,z,		-1,-1,0,	1,-1,0
/*    		
*/    		
            };

    public ES20SimpleCubeRenderer(Context context) 
    {
        ByteBuffer vbb = ByteBuffer.allocateDirect(mTriangleVerticesData.length
                * FLOAT_SIZE_BYTES);
        vbb.order(ByteOrder.nativeOrder());
        mFVertexBuffer = vbb.asFloatBuffer();
        mFVertexBuffer.put(mTriangleVerticesData);
        mFVertexBuffer.position(0);
    }

    protected void draw(GL10 gl, int positionHandle)
    {
        GLES20.glVertexAttribPointer(positionHandle, 3, GLES20.GL_FLOAT, false,
                0, mFVertexBuffer);
        checkGlError("glVertexAttribPointer maPosition");
        //mFVertexBuffer.position(TRIANGLE_VERTICES_DATA_UV_OFFSET);
        GLES20.glEnableVertexAttribArray(positionHandle);
        checkGlError("glEnableVertexAttribArray maPositionHandle");

        //this.trnslate(0,-1,0);
        this.rotate(16, 0,1,0.5f);
        this.setupMatrices();
        int vertexCount = mTriangleVerticesData.length/3;
        GLES20.glDrawArrays(GLES20.GL_TRIANGLES, 0, vertexCount);
        checkGlError("glDrawArrays");
    }
    @Override
    protected FrustumDimensions getFrustumDimensions() 
    {
    	return FrustumDimensions.getMedium();
    }
}
